//+------------------------------------------------------------------+
//|                                                 Laguerre RSI.mq4 |
//|                                Copyright  2005, David W. Thomas |
//|                                           mailto:davidwt@usa.net |
//+------------------------------------------------------------------+
// based on http://www.mesasoftware.com/TimeWarp.doc.
#property copyright "Copyright  2005, David W. Thomas"
#property link      "mailto:davidwt@usa.net"

#property indicator_separate_window
#property indicator_level2 0.75
#property indicator_level3 0.45
#property indicator_level4 0.15
#property indicator_minimum 0
#property indicator_maximum 1
#property indicator_buffers 1
#property indicator_color1 MediumPurple

//---- input parameters
extern double		Gamma		= 0.7;			//  
extern int			HMAPrd	= 5;				//   
//extern int			Counter	= 2000;			// -   
//extern int method_MA=3;
/*
metod_MA=
MODE_SMA 0    
MODE_EMA 1    
MODE_SMMA 2    
MODE_LWMA 3 -   
*/
//---- buffers
double RSI[];
double L0[];
double L1[];
double L2[];
double L3[];
double Temp1[];
double Temp2[];
double HMA[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   IndicatorBuffers(8);
//---- indicators
   SetIndexStyle(0, DRAW_LINE);
   SetIndexDrawBegin(0, 1);
	SetIndexLabel(0, "Laguerre RSI");
	SetIndexEmptyValue(0, -0.01);
   SetIndexBuffer(0, HMA);
   SetIndexBuffer(1, L0);
   SetIndexBuffer(2, L1);
   SetIndexBuffer(3, L2);
   SetIndexBuffer(4, L3);
   SetIndexBuffer(5, Temp1);
   SetIndexBuffer(6, Temp2);
   SetIndexBuffer(7, RSI);

//----
   string short_name="LaguerreRSI(" + DoubleToStr(Gamma, 2) + ")";
   IndicatorShortName(short_name);
   return(0);
}

//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
	int    limit;
	int    counted_bars = IndicatorCounted();
	double CU, CD;
	//---- last counted bar will be recounted
	if (counted_bars>0)
		counted_bars--;
	else
		counted_bars = 1;
	limit = Bars - counted_bars;
	//---- computations for RSI
	for (int i=limit; i>=0; i--)
	{
		L0[i] = (1.0 - Gamma)*Close[i] + Gamma*L0[i+1];
		L1[i] = -Gamma*L0[i] + L0[i+1] + Gamma*L1[i+1];
		L2[i] = -Gamma*L1[i] + L1[i+1] + Gamma*L2[i+1];
		L3[i] = -Gamma*L2[i] + L2[i+1] + Gamma*L3[i+1];
		
		CU = 0;
		CD = 0;
		if (L0[i] >= L1[i])
			CU = L0[i] - L1[i];
		else
			CD = L1[i] - L0[i];
		if (L1[i] >= L2[i])
			CU = CU + L1[i] - L2[i];
		else
			CD = CD + L2[i] - L1[i];
		if (L2[i] >= L3[i])
			CU = CU + L2[i] - L3[i];
		else
			CD = CD + L3[i] - L2[i];

		if (CU + CD != 0)RSI[i] = CU / (CU + CD);
				
	}
	int iCount=ArraySize(RSI)-HMAPrd;
	HMA_LWMAOnArray(RSI,HMA,HMAPrd,iCount);
	//for (i=limit-period_MA; i>=0; i--)RSI[i]=iMAOnArray(RSI,0,period_MA,0,method_MA,i);
	
   return(0);
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void LWMAOnArray(double aySource[],double& ayResult[],int iPeriod,int iCount)
{
	for (int Ax=iCount-1;Ax>=0;Ax--){
		double Sum = 0;
		double Weight = 0;
		for (int Px = 0;Px < iPeriod;Px++){ 
			Weight=Weight+ (iPeriod - Px);
			Sum = Sum+aySource[Ax+Px]*(iPeriod - Px);
		}
		if(Weight>0) ayResult[Ax] = Sum/Weight;
		else ayResult[Ax] = 0; 
	}
	return;
} 
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void HMA_LWMAOnArray(double aySource[],double& ayResult[],int iPeriod,int iCount)
{
	LWMAOnArray(aySource,Temp1,MathFloor(iPeriod/2),iCount);
	LWMAOnArray(aySource,Temp2,iPeriod,iCount);
	for (int Ax=iCount-1;Ax>=0;Ax--) Temp1[Ax]=2.0*Temp1[Ax]-Temp2[Ax];
	LWMAOnArray(Temp1,ayResult,MathFloor(MathSqrt(iPeriod)),iCount);
 	return;
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

